/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

#include "Sensorless.h"
#include "Filters10KHz.h"	// TODO: PABLO: This include should be avoided at all

volatile struct SensorlessConfiguration sensorlessConf;

/********************** PLL variables **********************************/
volatile data h_erp;			// estimated rotor harmonic order position
volatile data delta_erp;		// increment in estimated rotor position
volatile data error_erp;		// estimated rotor position error
//volatile data erp;				// estimated rotor position
/************************************************************************/


/********************** Estimation Filter variables *********************/
//_iq BO_LPF,A1_LPF;				// lpf filter
_iq BO_HPF,A1_HPF;				// hpf filter
//_iq Kp_TO;						// bandwidth
/************************************************************************/


/********************** PULSATING CARRIER VARIABLES *********************/
volatile struct sqddata vs_e_c;
volatile struct qddata iqds_e_puls;	// <-- temp
volatile struct qddata iqds_s_hpf_puls;
volatile struct qddata iqds_s_hpf_hpf_puls;
volatile struct qddata iqds_ce;
volatile struct qddata iqds_ce_lpf;
volatile struct qddata iqds_e_lpf_hb;
volatile struct qddata iqds_nce;
volatile struct qddata iqds_nce_lpf;
volatile struct qddata iqds_e_lpf_lb;
volatile struct qddata iqds_e_lpf;
volatile struct qddata iqds_e_lpf_ca;
volatile struct qddata iqds_e_lpf_ca_lpf;
volatile data theta_hat_error;
volatile data wr_hat_pulsC;
_iq Kp_Puls,Ki_Puls;
volatile _iq iqds_e_lpf_q;
/*************************************************************************/

//-----------------------------------------------------------------------------
//	Sensorless Functions
//-----------------------------------------------------------------------------

void FillCarrierTable(_iq* carrierTable, const Uint16 carrierPoints, 
	const Uint16 synchoCycles)
{
	Uint16 carrierIndx;
	_iq carrierAngleInc;
	sensorlessConf.carrierDataTable = carrierTable;
	carrierAngleInc=_IQdiv(TWOPI,_IQ(carrierPoints));
	for(carrierIndx=0; carrierIndx<carrierPoints; carrierIndx++)
		sensorlessConf.carrierDataTable[carrierIndx]=
			_IQmpy(_IQ(carrierIndx),carrierAngleInc);
	sensorlessConf.carrierIndx = 0;
	sensorlessConf.carrierPoints = carrierPoints;
	sensorlessConf.carrierSynchro = carrierPoints*synchoCycles;
}

//-----------------------------------------------------------------------------

void InitPLL(_iq dTDiv2, _iq saliencyOrder, _iq BO_LPF, _iq A1_LPF)
{
	//Kp_TO=_IQ(10000);
	//BO_LPF=_IQmpy(DTDIV2,Kp_TO);
	sensorlessConf.BO_LPF = BO_LPF;//_IQ(0.1); // <-- DTDIV2*Kp_TO
	sensorlessConf.A1_LPF = A1_LPF;//_IQ(1);
	sensorlessConf.dTDiv2 = dTDiv2;
	sensorlessConf.saliencyOrder = saliencyOrder;
	INIT_DATA(h_erp);
	INIT_DATA(delta_erp);
	INIT_DATA(error_erp);
	//INIT_DATA(erp);
}

//-----------------------------------------------------------------------------

void InitPulsatingCarrierDemodulator(_iq dTDiv2, _iq BO_LPF, _iq A1_LPF,
	 _iq BO_HPF, _iq A1_HPF)
{
	sensorlessConf.dTDiv2 = dTDiv2;
	sensorlessConf.BO_LPF = BO_LPF;//B0_LPF100;
	sensorlessConf.A1_LPF = A1_LPF;//A1_LPF100;
	sensorlessConf.BO_HPF = BO_HPF;//B0_HPF100;
	sensorlessConf.A1_HPF = A1_HPF;//A1_HPF100;
	Kp_Puls=_IQmpy(TWOPI,_IQ(200));
	Ki_Puls=_IQ(2);
 	INIT_QDDATA(iqds_s_hpf_puls);
	INIT_QDDATA(iqds_s_hpf_hpf_puls);
	INIT_QDDATA(iqds_e_puls);	// <-- temp
	INIT_QDDATA(iqds_ce);
	INIT_QDDATA(iqds_ce_lpf);
	INIT_QDDATA(iqds_e_lpf_hb);
	INIT_QDDATA(iqds_nce);
	INIT_QDDATA(iqds_nce_lpf);
	INIT_QDDATA(iqds_e_lpf_lb);
	INIT_QDDATA(iqds_e_lpf);
	INIT_QDDATA(iqds_e_lpf_ca);
	INIT_QDDATA(iqds_e_lpf_ca_lpf);
	INIT_DATA(theta_hat_error);
	INIT_DATA(wr_hat_pulsC);
}

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------
