/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

/**\file**********************************************************************
 File Name	:	Sensorless.h
 Project	:  	TIMotorLIB
 
 TI Doc:		na
 \author  		Pablo Garcia (pgarcia@isa.uniovi.es)
 \date 			2007	     
 \brief 		Carrier based sensorless operation AC machines
******************************************************************************/

//-----------------------------------------------------------------------------
//	Define
//-----------------------------------------------------------------------------
#ifndef SENSORLESS_H
#define SENSORLESS_H

//-----------------------------------------------------------------------------
//	Includes
//-----------------------------------------------------------------------------
#include "DSP2833x_Device.h"
#include "Constants.h"
#include "QDOperations.h"
#include "DataStorage.h"
#include "FilterAndRotations.h"
#include "Utils.h"


/*// carrier frequencies
#define FC500				(_IQ(0.2))
#define FC50				(_IQ(0.02))
#define FC5					(_IQ(0.002))
// carrier step angle
#define ANGFC2500			(_IQ(1.5709632679489661923))
#define ANGFC500			(_IQ(0.6283185307179586))
#define ANGFC50				(_IQ(0.06283185307179586))
#define ANGFC5				(_IQ(0.006283185307179586))
*/

//-----------------------------------------------------------------------------
//	Implemented Functions
//-----------------------------------------------------------------------------
extern void FillCarrierTable(_iq* carrierTable, 
	const Uint16 carrierPoints, const Uint16 synchoCycles);
extern void InitPLL(_iq dTDiv2, _iq saliencyOrder, _iq BO_LPF, _iq A1_LPF);
extern void InitPulsatingCarrierDemodulator(_iq dTDiv2, _iq BO_LPF, _iq A1_LPF,
	 _iq BO_HPF, _iq A1_HPF);
static inline _iq CarrierAngle(void);
static inline void RotatingCarrierVoltage(volatile _iq* Vhf, 
	volatile struct sqddata* vs_s_c, volatile _iq* theta_c);
static inline void PulsatingCarrierVoltage(volatile _iq* Vhf,
	volatile struct sqddata* vs_s_c, volatile _iq* theta_c, volatile _iq* erp);
static inline void PLL(struct qddata iqds_n, volatile data* erp);
static inline void DemodulatePulsatingCarrier(volatile struct qddata* iqds_s,volatile data* erp,
	volatile _iq* theta_hat, volatile _iq theta_c);
/*static inline void AdaptiveTransientImpedanceEstimation(volatile struct qddata* iqds_pc_lpf,
	volatile struct qddata* iqds_nc_lpf);
*/


//-----------------------------------------------------------------------------
//	Variables
//-----------------------------------------------------------------------------
// variables in sensorless.c
struct SensorlessConfiguration{
	_iq* carrierDataTable;		///< pointer to carrierTable data
	Uint16 carrierPoints;		///< number of carrier points
	Uint16 carrierIndx;			///< actual pos in carrier table
	Uint16 carrierSynchro;		///< synchonize table every carrierSynchro cycles
	_iq dTDiv2;					///< sample time divided by 2
	_iq saliencyOrder;			///< saliency order for estimation (PLL)
	_iq BO_LPF,A1_LPF;			///< low pass filter coefficients for estimation
	_iq BO_HPF,A1_HPF;			///< high pass filter coefficients for estimation
};

extern volatile struct SensorlessConfiguration sensorlessConf;


//#ifdef LIBRARY_BUILD // variables only visible during libraty building
// extern variables 
//extern volatile _iq theta_c_rot;
extern volatile data h_erp;			// estimated rotor harmonic order position
extern volatile data delta_erp;		// increment in estimated rotor position
extern volatile data error_erp;		// estimated rotor position error
extern volatile data erp;			// estimated rotor position


//extern _iq BO_LPF,A1_LPF;	// lpf filter
//extern _iq BO_HPF,A1_HPF;	// hpf filter
/********************** PULSATING CARRIER VARIABLES *********************/
//extern volatile _iq theta_c_rot;
extern volatile struct sqddata vs_e_c;
extern volatile struct qddata iqds_e_puls;	// <-- temp
extern volatile struct qddata iqds_s_hpf_puls;
extern volatile struct qddata iqds_s_hpf_hpf_puls;
extern volatile struct qddata iqds_ce;
extern volatile struct qddata iqds_ce_lpf;
extern volatile struct qddata iqds_e_lpf_hb;
extern volatile struct qddata iqds_nce;
extern volatile struct qddata iqds_nce_lpf;
extern volatile struct qddata iqds_e_lpf_lb;
extern volatile struct qddata iqds_e_lpf;
extern volatile struct qddata iqds_e_lpf_ca;
extern volatile struct qddata iqds_e_lpf_ca_lpf;
extern volatile data theta_hat_error;
extern volatile data wr_hat_pulsC;
extern _iq Kp_Puls,Ki_Puls;
extern volatile _iq iqds_e_lpf_q;
/*************************************************************************/
//#endif


// ----------------------------------------------------------------------------
/// Generate carrier angle
// ----------------------------------------------------------------------------
static inline _iq CarrierAngle(void)
{
	_iq theta_c=sensorlessConf.carrierDataTable[
		sensorlessConf.carrierIndx%sensorlessConf.carrierPoints];
	if(++sensorlessConf.carrierIndx==sensorlessConf.carrierSynchro)
		sensorlessConf.carrierIndx=0;
	return theta_c;
}

//-----------------------------------------------------------------------------


// ----------------------------------------------------------------------------
/// Generate a rotating carrier vector voltage in dqs coordinates
// ----------------------------------------------------------------------------
static inline void RotatingCarrierVoltage(volatile _iq* Vhf,
	volatile struct sqddata* vs_s_c, volatile _iq* theta_c)
{
	*theta_c = CarrierAngle();
	SSCALAR2SYNCVEC(*Vhf,(*vs_s_c),(*theta_c));
}

//-----------------------------------------------------------------------------


// ----------------------------------------------------------------------------
/// Generate a pulsating carrier vector voltage in dqs coordinates
// ----------------------------------------------------------------------------
static inline void PulsatingCarrierVoltage(volatile _iq* Vhf,
	volatile struct sqddata* vs_s_c, volatile _iq* theta_c, volatile _iq* erp)
{
	struct sqddata vs_e_c;
	*theta_c = CarrierAngle();
	SSCALAR2PULSATING(*Vhf,vs_e_c,(*theta_c));
	SROTATE_NEG(vs_e_c,(*vs_s_c),(*erp));
}

//-----------------------------------------------------------------------------


// ----------------------------------------------------------------------------
/// Demodulate rotating carrier voltage using a PLL
// ----------------------------------------------------------------------------
static inline void PLL(struct qddata iqds_n, volatile data* erp)
{

	// estimated h order saliency position
   	h_erp[N]=h_erp[N1]+_IQmpy(sensorlessConf.saliencyOrder,delta_erp[N1]);

   	WRAP2PI(h_erp[N]);   

	// anti-alias filter lag compensation
	//ROTATE_NEG(iqds_n,iqds_n,ANTIALIAS_FILTER_LAG);
   	// PLL implementation
   	CROSS_QDDATA(iqds_n,h_erp[N],error_erp[N]);//error_erp[N]=imag(Vqd_ca[N])*cos(h_erp[N])-real(Vqd_ca[N])*sin(h_erp[N]);
   
   	WRAP2PI(error_erp[N]);

   	// estimated rotor position (LPF filtered)
	FILTER_LPF(error_erp,(*erp),sensorlessConf.BO_LPF,sensorlessConf.A1_LPF);//erp[N]=erp[N1]+DTDIV2*Kp_TO*(error_erp[N]+error_erp[N1]);
	// anti-alias filter lag compensation
	// erp[N]+=ANTIALIAS_FILTER_LAG;

   	WRAP2PI((*erp)[N]);
   	// increment in rotor position
   	delta_erp[N]=*erp[N]-*erp[N1];

  	WRAP2PI(delta_erp[N]);   
   
	UPDATE_STATE(h_erp);
	UPDATE_STATE(delta_erp);
	UPDATE_STATE(error_erp);
	//UPDATE_STATE(erp);

	//return erp[N];
}

//-----------------------------------------------------------------------------


// ----------------------------------------------------------------------------
/// Demodulate pulsating carrier voltage
// ----------------------------------------------------------------------------
static inline void DemodulatePulsatingCarrier(volatile struct qddata* iqds_s,
	volatile data* erp, volatile _iq* theta_hat, volatile _iq theta_c)
{
	// low-pass filter high-band
	//_iq saliency_angle=theta_c_rot - _IQmpy(SALIENCY_ORDER,*(erp[N]));	// <-- use ERP to sensorless
	// _iq saliency_angle=theta_c_rot - _IQmpy(SALIENCY_ORDER,*(theta_hat));
	FILTER_QD_HPF((*iqds_s),iqds_s_hpf_puls,sensorlessConf.BO_HPF,sensorlessConf.A1_HPF);
	FILTER_QD_HPF(iqds_s_hpf_puls,iqds_s_hpf_hpf_puls,sensorlessConf.BO_HPF,sensorlessConf.A1_HPF);
	ROTATE_POS(iqds_s_hpf_hpf_puls,iqds_e_puls, (*theta_hat));

	ROTATE_POS(iqds_e_puls,iqds_ce,theta_c);	// <-- temp
	//ROTATE_POS((*iqds_s),iqds_ce,(theta_c + (*theta_hat) ) );
	FILTER_QD_LPF(iqds_ce,iqds_ce_lpf,sensorlessConf.BO_LPF,sensorlessConf.A1_LPF);
	ROTATE_NEG(iqds_ce_lpf,iqds_e_lpf_hb,theta_c);

	// low-pass filer low_band
	ROTATE_NEG(iqds_e_puls,iqds_nce,theta_c);	// <-- temp
	//ROTATE_NEG((*iqds_s),iqds_nce,(theta_c - (*theta_hat) ) );
	FILTER_QD_LPF(iqds_nce,iqds_nce_lpf,sensorlessConf.BO_LPF,sensorlessConf.A1_LPF);
	ROTATE_POS(iqds_nce_lpf,iqds_e_lpf_lb,theta_c);

	// easier to filter fundamental or complex filtering
	
	// add both components
	SUM_QD_DATA(iqds_e_lpf_hb,iqds_e_lpf_lb,iqds_e_lpf);

	// take imaginary part
	iqds_e_lpf_q=iqds_e_lpf.q[N];	// <-- not used
	// select low-band (high-band can be used also using ROTATE_POS_ESC)
	ROTATE_NEG_ESC(iqds_e_lpf.q[N],iqds_e_lpf_ca,theta_c);
	// filter 2wc components
	FILTER_QD_LPF(iqds_e_lpf_ca,iqds_e_lpf_ca_lpf,sensorlessConf.BO_LPF,sensorlessConf.A1_LPF);
	// also a hpf filtering? (see matlab script PulsatingCarrierPositionEstimation)

	// position estimation
	theta_hat_error[N]=iqds_e_lpf_ca_lpf.q[N];					// error signal is q component
	// <-- HOLTZ
	//if(iqds_e_lpf_ca_lpf.d[N]<0.0)
	//	theta_hat_error[N]=-theta_hat_error[N];
	PI_REG_DT(theta_hat_error,wr_hat_pulsC,Kp_Puls,Ki_Puls,sensorlessConf.dTDiv2);		// PI output estimated velocity
	INT_DT(wr_hat_pulsC,(*erp),sensorlessConf.dTDiv2);											// integral to position

	WRAP2PI((*erp)[N]);

 	UPDATEQD_STATE(iqds_s_hpf_puls);
	UPDATEQD_STATE(iqds_s_hpf_hpf_puls);
	UPDATEQD_STATE(iqds_e_puls);	// <-- temp
	UPDATEQD_STATE(iqds_ce);
	UPDATEQD_STATE(iqds_ce_lpf);
	UPDATEQD_STATE(iqds_e_lpf_hb);
	UPDATEQD_STATE(iqds_nce);
	UPDATEQD_STATE(iqds_nce_lpf);
	UPDATEQD_STATE(iqds_e_lpf_lb);
	UPDATEQD_STATE(iqds_e_lpf);
	UPDATEQD_STATE(iqds_e_lpf_ca);
	UPDATEQD_STATE(iqds_e_lpf_ca_lpf);
	UPDATE_STATE(theta_hat_error);
	UPDATE_STATE(wr_hat_pulsC);

}

//-----------------------------------------------------------------------------



// ----------------------------------------------------------------------------
// Method to estimate transient impedance of the machine at carrier frequency
// ----------------------------------------------------------------------------
/*static inline void AdaptiveTransientImpedanceEstimation(volatile struct qddata* iqds_pc_lpf,
	volatile struct qddata* iqds_nc_lpf)
{
//	_IQdiv()
}*/

//-----------------------------------------------------------------------------

#endif

//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------

