/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

/**\file**********************************************************************
 File Name	:	TIMotorLIB.h
 Project	:  	TIMotorLIB
 
 TI Doc:		<a href="http://www.ti.com/litv/pdf/spru566i"> SPRU566</a> 
 \author  		Pablo Garcia (pgarcia@isa.uniovi.es)
 \date 			2007	     
 \brief 		TIMotorLIB main header file
 \version		0.3
******************************************************************************/

/*! \mainpage TIMotorLIB
 *
The aim of this document is to introduce TIMotorLIB. TIMotorLIB is a open source library in the terms of LGPL license. The proposal of this library is to achieve fast development of digital control systems, mostly focused on the control of power converters, using Texas Instruments TMS320F28XXX Digital Signal Controllers (DSC) based boards. The development of this code has been done using the eZdsp F28335 board from Spectrum Digital.

The library is programmed using C as the main language, although additional contents can be developed using TMS320C28x assembler (F28335 CPU).

An overview of C2000 Real-Time MCUs can be found <a href="http://focus.ti.com/mcu/docs/mcuproductcontentnp.tsp?sectionId=95&familyId=916&tabId=2656&DCMP=digpwr&HQS=Other+BA+digpwr_bhp"> here</a>

This manual is organized in the following sections:

- \subpage pagHardw
- \subpage pagSoft
- \subpage pagInstallation
- \subpage pagUser
- \subpage pagExamples
- \subpage pagLicense
- \subpage pagCodingStyle

<a href="http://isa.uniovi.es/ISAwiki/index.php/TIMotorLIB">Wiki page of the project</a>
 */

/*! \page pagHardw Hardware related units
    \section CPU CPU clocking and interrupts
	\copybrief cpu.h cpu.h
    \section GPIO General purpose input output (GPIO) 
	\copybrief gpio.h gpio.h
    \section ADC Analog to digital converter (ADC) 
	\copybrief adc.h adc.h    
    \section EPWM Enhanced pulse width modulator (EPWM) 
	\copybrief Epwm.h Epwm.h    
    \section QEP Quadrature encoder peripheral (QEP) 
	\copybrief qep.h qep.h    
    \section SCI Serial communications interface (SCI) 
	\copybrief sci.h sci.h    
    \section MEM Memory data storage
	\copybrief DataStorage.h DataStorage.h
 */

/*! \page pagSoft Software related units
    \section Complex Complex numbers operations
	\copybrief QDOperations.h QDOperations.h
    \section MotorControl Motor control routines 
	\copybrief Control.h Control.h
    \section DSP Digital signal processing routines
	\copybrief FilterAndRotations.h FilterAndRotations.h
 */

/*! \page pagInstallation Installation manual
 * TIMotorLIB can be used with Code Composer 3.3 and 4.0 versions. Creation steps for a project using the library in both IDE's is described below.
    \section TIMotorLIBCompiling Compiling the library
	    \li Add Texas Headers (downloadable from <a href="http://focus.ti.com/docs/toolsw/folders/print/sprc530.html">here</a>) directory to compiler include path:
  
		C:\\tidcs\\c28\\DSP2833x\\vXXX\\DSP2833x\\headers\\include
		
		C:\\tidcs\\c28\\DSP2833x\\vXXX\\DSP2833x\\common\\include
		
		C:\\tidcs\\c28\\DSP2833x\\vXXX\\DSP2833x\\common\\source
	\subsection TIMotorLIBConf project configuration
	    \li Project --> Properties --> Basic Options: Optimization level (O2 (default)).
	    \li Project --> Properties --> C2000 Compiler: Runtime Model Options: Check Use large memory model and Unified memory model and uncheck Specify floating point support (--float support).
    \section TIMotorLIBUsing Using the library
    \subsection CCv3 Code Composer 3.3
	    \li Project --> New...
	    \li Choose a \a Project \a Name, a \a Location, set \a Project \a Type to \a Executable (.out) and \a Target to TMS320C28XX
	    \li Open Project --> Build Options
	    \li In Project --> Build Options --> Compiler --> Basic, set \a Opt \a Level to \a Function (-o2)
	    \li In Project --> Build Options --> Compiler --> Advanced, set Use large memory model and Unified memory model.
	    \li In Project --> Build Options --> Compiler --> Preprocessor add to the \a Include \a Search \a Path TIMotorLIB directory and Texas Headers (downloadable from <a href="http://focus.ti.com/docs/toolsw/folders/print/sprc530.html">here</a>) directory to compiler include path:
  
		C:\\tidcs\\c28\\DSP2833x\\vXXX\\DSP2833x\\headers\\include
		
		C:\\tidcs\\c28\\DSP2833x\\vXXX\\DSP2833x\\common\\include
		
		C:\\tidcs\\c28\\DSP2833x\\vXXX\\DSP2833x\\common\\source
		
	    \li In Project --> Build Options --> Compiler --> Preprocessor  Pre-Define needed library symbols: FIXED_POINT_LIB, D_AXIS_REAL or Q_AXIS_REAL and USE_XINTF
	    \li In Project --> Build Options --> Linker --> Basic, set \a Stack \a Size to 0x380.
	    \li In Project --> Build Options --> Linker --> Libraries, add \a C:\\CCStudio_v3.3\\C2000\\cgtools\\lib to search path and include \a rts2800_ml.lib.
	    \li Project --> Add Files to Project. Object and Library Files: \a TIMotorLIB.lib, \a IQMath.lib and \a SFO_TI_Build_V5.lib
	    \li Project --> Add Files to Project. Linker Command File: \a TIMotorLIB.cmd
    \subsection CCv4 Code Composer 4.0
	\subsubsection ProjectWizard project wizard
	    \li Choose a project name.
	    \li Select both Debug/Release configurations.
	    \li Link Linker Command File: TIMotorLIB.cmd in the TIMotorLIB folder.
	\subsubsection ProjectConf project configuration
	    \li Project --> Add Files to Project. Object and Library Files: \a TIMotorLIB.lib, \a IQMath.lib and \a SFO_TI_Build_V5.lib	
	    \li Project --> Properties --> Basic Options: Optimization level (O2 (default)).
	    \li Project --> Properties --> C2000 Compiler: Runtime Model Options: Check Use large memory model and Unified memory model and uncheck Specify floating point support (--float support).
	    \li Project --> Properties --> C2000 Linker: File Search Path: Include library or command file as input: rts2800\_ml.lib.
	    \li Project --> Properties --> C2000 Linker: Basic Options: Set C System stack size: 0x380.
	    \li Add TIMotorLIB directory to compiler include path (Project --> Properties --> Tool Settings: C/C++ Build --> Include Options: Workspace: TIMotorLIB
	    \li Add Texas Headers (downloadable from <a href="http://focus.ti.com/docs/toolsw/folders/print/sprc530.html">here</a>) directory to compiler include path:
  
		C:\\tidcs\\c28\\DSP2833x\\vXXX\\DSP2833x\\headers\\include
		
		C:\\tidcs\\c28\\DSP2833x\\vXXX\\DSP2833x\\common\\include
		
		C:\\tidcs\\c28\\DSP2833x\\vXXX\\DSP2833x\\common\\source
		
    \section Configuration Library Configuration
    TIMotorLIB can be configured some of its parameters by means of preprocessor language These macros can be defined at each project using the library in the Project --> Properties --> C2000 Compiler --> Predefined Symbols. Used macros are following described:
	- DSP_FAMILY: This macro is used to change Texas Headers includes depending on family selection. By default is set to DSP2833x 
	- LIBRARY_BUILD: This macro is used to report errors at compiling time if needed symbols have not been defined. Definition of
	    this macro asserts that error reporting only happens when a file is compiled to generate an executable. 
	    THIS SYMBOL MUST BE DEFINED WHEN COMPILING TIMOTORLIB.
	- FIXED_POINT_LIB/FLOATING_POINT_LIB: The library has been designed with the aim of working both in fixed point DSCs 
	    (using IQMath) and in floating point DSCs. ONE OPTION MUST BE DEFINED WHEN COMPILING TIMOTORLIB. \note Floating point behavior
	    needs to be tested.
	- D_AXIS_REAL/Q_AXIS_REAL: Three phase systems are best analyzed when represented as a complex variable (ab = a+jb). When dealing
	    with complex variables, an axis orientation must be chosen. There are two common used options dq = d+jq (D_AXIS_REAL) or 
	    qd = q-jd (Q_AXIS_REAL). ONE OPTION MUST BE DEFINED WHEN COMPILING TIMOTORLIB. 
	    be selected by definin
	- USE_XINTF/USE_FLASH: DataStorage unit is used for data storing in memory. The library can be configured for using an external
	    memory connected through the external interface (USE_XINTF) or flash memory (USE_FLASH). 
	    ONE OPTION MUST BE DEFINED WHEN COMPILING TIMOTORLIB. \note Flash is still not working in version 0.3
 */

/*! \page pagUser User manual
 * User manual is provided as a separated UserManual.pdf document in the Doc folder.
 */

/*! \page pagExamples Examples
 * Examples are provided with this library in the Examples folder.
 * \example TIMotorLIBExample1/main.c
 * This example shows how to develop a simple program with a main loop an one interrupt routine service for A/D. In the interrupt routine service, $i_a$ and $i_b$ currents are read.
 * \example TIMotorLIBExample2/main.c 
 * This example adds PWM generation for V/HZ control to Example 1.
 * \example TIMotorLIBExample3/main.c 
 * This example adds current vector control to Example 2.
 * \example TIMotorLIBExampleHrdw/main.c
 * This is an example on how to use hardware units.
 * \example TIMotorLIBExampleSw/main.c
 * This is an example on how to use some of the control and signal processing utilities.
 */

/*! \page pagLicense License
 * Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * */

/*! \page pagCodingStyle Coding Style
 * \verbinclude coding_style.txt
 */

//-----------------------------------------------------------------------------
//	Define
//-----------------------------------------------------------------------------

#ifndef TIMOTORLIB_H
#define TIMOTORLIB_H

/// \def DSP_FAMILY macro for generation of Texas Headers includes
#ifndef DSP_FAMILY
	#define DSP_FAMILY			DSP2833x
#endif
#define JOIN(x, y) 					JOIN_AGAIN(x, y)
#define JOIN_AGAIN(x, y) 			x ## y
#define QUOTE(PARAM)				#PARAM
#define MACRO_INC(PARAM)			QUOTE(PARAM)
/*! \def GENERATE_FLEX_INC(FILE) 
 * this macro SHOULD be used to include Texas Headers files depending on 
 * DSP_FAMILY selection. Example: GENERATE_FLEX_INC(Device.h) */
#define GENERATE_FLEX_INC(FILE)		MACRO_INC(JOIN(DSP_FAMILY,_##FILE))	

/// \typedef interruptFCN pointer to Interrupt Service Routine (ISR)
typedef interrupt void(*interruptFCN)(void);

// Hardware headers
#include "cpu.h"			///< PLL, WatchDog and systemCtrl utilities
#include "cpuTimers.h"		///< CPU Timers
#include "gpio.h"			///< General purpose input/output
#include "Epwm.h"			///< PWM routines
#include "adc.h"			///< analog to digital routines
#include "qep.h"			///< encoder routines
#include "sci.h"			///< serial routines
// Software headers
#include "Constants.h"		///< trigonometric constants
#include "QDOperations.h"	///< complex number operations
#include "Utils.h"			///< utils
#include "Control.h"		///< control routines
#include "Sensorless.h"		///< Sensorless routines

#endif

//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------

