/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

/**\file**********************************************************************
 File Name	:	Utils.h
 Project	:  	TIMotorLIB
 
 TI Doc:		na
 \author  		Pablo Garcia (pgarcia@isa.uniovi.es)
 \date 			2007
 \brief 		General purpose macros
******************************************************************************/

//-----------------------------------------------------------------------------
//	Define
//-----------------------------------------------------------------------------

#ifndef UTILS_H
#define UTILS_H

// ----------------------------------------------------------------------------
// Includes
// ----------------------------------------------------------------------------

#include "IQConstants.h"


//-----------------------------------------------------------------------------
//  MACROS declaration
//-----------------------------------------------------------------------------

#ifdef MAX
#undef MAX
#endif
#ifdef MIN
#undef MIN
#endif
#ifdef ABS
#undef ABS
#endif
/// returns the maximum of A, B
#define	MAX(A, B)           ((A) >= (B) ? (A) : (B))
/// returns the minimum of A, B
#define	MIN(A, B)           ((A) <= (B) ? (A) : (B))


/// returns the absolute value of C
#define	ABS(C)              ((C) >= (0) ? (C) : (-C))
/// reassigns A so that it is limited by +/- L (arg +L)
#define LIM(A,L) 			(A)=MIN(MAX(A,-L),L)
/// wrap max angle to ANG
#define WRAPSUP(A,ANG)			((A) > (ANG) ? (A-_IQmpy(ANG,2))	: (A))
/// wrap min angle to ANG
#define WRAPINF(A,ANG)			((A) < (ANG) ? (A+_IQmpy(ANG,2))	: (A))
/// wrap min, max angle to ANG
#define WRAP(A,ANG)				(A) = WRAPINF(WRAPSUP(A,ANG),-ANG)

/// wrap max angle to PI
#define WRAPSUP2PI(A)		((A) > (PI)  ? (A-TWOPI)	: (A))
/// wrap min angle to PI
#define WRAPINF2PI(A)		((A) < (-PI) ? (A+TWOPI)	: (A))
/// wrap min, max angle to PI
#define WRAP2PI(A)			(A) = WRAPINF2PI(WRAPSUP2PI(A))

//-----------------------------------------------------------------------------
//	Implemented Functions
//-----------------------------------------------------------------------------
static inline _iq GetMax(volatile _iq* a, volatile _iq* b, volatile _iq** ptrMax);
static inline _iq GetMin(volatile _iq* a, volatile _iq* b, volatile _iq** ptrMin);

//-----------------------------------------------------------------------------
//	Inline function implementation
//-----------------------------------------------------------------------------

/// return the maximum of two numbers and a pointer to the address of maximum variable
///
///	\param a: pointer to first number
///	\param b: pointer to second number
///	\param ptrMax: address of the maximum variable
/// RETURNS:
///	\return: maximum value
static inline _iq GetMax(volatile _iq* a, volatile _iq* b, volatile _iq** ptrMax)
{
    _iq c;
    if(*a >= *b) {
		c = *a;
		*ptrMax = a;
    }
    else{
		c = *b;
		*ptrMax = b;	
    }
    return c;
}

//-----------------------------------------------------------------------------

/// return the minimum of two numbers and a pointer to the address of minimum variable
///	\param a: pointer to first number
///	\param b: pointer to second number
///	\param ptrMin: address of the minimum variable
///	\return: minimum value
static inline _iq GetMin(volatile _iq* a,volatile  _iq* b, volatile _iq** ptrMin)
{
    _iq c;
    if(*a <= *b) {
		c = *a;
		*ptrMin = a;
    }
    else{
		c = *b;
		*ptrMin = b;	
    }
    return c;
}

//-----------------------------------------------------------------------------

#endif

//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------
