/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

//-----------------------------------------------------------------------------
//	Includes
//-----------------------------------------------------------------------------
#include "cpuTimers.h"
#include "cpu.h"

//-----------------------------------------------------------------------------
//	variables
// ----------------------------------------------------------------------------
// CPUTimers data structures
struct CPUTIMER_VARSIQ TIMLCpuTimer0;
struct CPUTIMER_VARSIQ TIMLCpuTimer1;
struct CPUTIMER_VARSIQ TIMLCpuTimer2;

// ----------------------------------------------------------------------------

void InitCpuTimers(void)
{
	InitCpuTimer(CPUTIMER0);
	InitCpuTimer(CPUTIMER1);
	InitCpuTimer(CPUTIMER2);
}

// ----------------------------------------------------------------------------

void InitCpuTimer(CPUTIMERSEnum cpuTimer)
{
	volatile struct  CPUTIMER_REGS  *RegsAddr = 0;
	switch(cpuTimer){
		case CPUTIMER0:
			TIMLCpuTimer0.RegsAddr =  RegsAddr = &CpuTimer0Regs;
			break;
		case CPUTIMER1:
			TIMLCpuTimer1.RegsAddr =  RegsAddr = &CpuTimer1Regs;
			break;
		case CPUTIMER2:
			TIMLCpuTimer2.RegsAddr =  RegsAddr = &CpuTimer2Regs;
			break;
	}
	// Initialize timer period to maximum:
	RegsAddr->PRD.all  = 0xFFFFFFFF;
	// Initialize pre-scale counter to divide by 1 (SYSCLKOUT):
	RegsAddr->TPR.all  = 0;
	RegsAddr->TPRH.all = 0;
	// Make sure timer is stopped:
	RegsAddr->TCR.bit.TSS = 1;
	// Reload all counter register with period value:
	RegsAddr->TCR.bit.TRB = 1;
}

// ----------------------------------------------------------------------------

void ConfigureCpuTimer(CPUTIMERSEnum cpuTimer, Uint32 Period,  void(*interruptFCN)(void))
{
	struct CPUTIMER_VARSIQ *Timer = 0;
	InitCpuTimer(cpuTimer);
	switch(cpuTimer){
		case CPUTIMER0:
			Timer = &TIMLCpuTimer0;
			break;
		case CPUTIMER1:
			Timer = &TIMLCpuTimer1;
			break;
		case CPUTIMER2:
			Timer = &TIMLCpuTimer2;
			break;
	}
	// Initialize timer period:
	Timer->CPUFreqInMHz = CPUINMHZ();
	Timer->PeriodInUSec = Period;
	Timer->RegsAddr->PRD.all = (long) (Timer->CPUFreqInMHz * Timer->PeriodInUSec);

	// Set pre-scale counter to divide by 1 (SYSCLKOUT):
	// PABLO: maximum timer value with 75Mhz SYSCLKOUT = (2^32-1)/75000000 = 57.3 s
	Timer->RegsAddr->TPR.all  = 0;
	Timer->RegsAddr->TPRH.all  = 0;

	// Initialize timer control register:
	Timer->RegsAddr->TCR.bit.TSS = 1;      	// 1 = Stop timer, 0 = Start/Restart Timer
	Timer->RegsAddr->TCR.bit.TRB = 1;      	// 1 = reload timer
	Timer->RegsAddr->TCR.bit.SOFT = 0;
	Timer->RegsAddr->TCR.bit.FREE = 0;     	// Timer Free Run Disabled
	Timer->RegsAddr->TCR.bit.TIE = 0;		// Disable Timer Interrupt
	if(interruptFCN != 0){
		Timer->RegsAddr->TCR.bit.TIE = 1;     	// Enable Timer Interrupt
		// ISR
		switch(cpuTimer){
			case CPUTIMER0:
				EALLOW;  
   				PieVectTable.TINT0 = interruptFCN;
   				EDIS;
				// Enable CPU INT1 which is connected to TIMER0 INT:
   				IER |= M_INT1;
				PieCtrlRegs.PIEIER1.bit.INTx7 = 1;	// TIMER0
				break;
			case CPUTIMER1:
				// Enable CPU INT13 which is connected to TIMER1 INT:
   				IER |= M_INT13;
				// PABLO: not connected to PIE --> (spru566d.pdf)
				break;
			case CPUTIMER2:
				EALLOW;  
   				PieVectTable.TINT2 = interruptFCN;
   				EDIS;
				// Enable CPU INT14 which is connected to TIMER2 INT:
   				IER |= M_INT14;
				// PABLO: not connected to PIE --> (spru566d.pdf)
				break;
		}
    }
}

// ----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------

