/******************************************************************************
** Copyright (C) 2007  Pablo Garcia
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/



// ----------------------------------------------------------------------------
// Includes
// ----------------------------------------------------------------------------
#include "qep.h"
#include "cpu.h"
//#include "motorConstants.h"


// ----------------------------------------------------------------------------
// Variables
// ----------------------------------------------------------------------------
volatile struct QEP qep1;
volatile struct QEP qep2;

// ----------------------------------------------------------------------------
// Functions declaration
// ----------------------------------------------------------------------------
void InitQEP1(void(*interruptFCN)(void), Uint16 qep_us, PCRM pcrm, 
	Uint16 lines, Uint16 polePairs);
void InitQEP2(void(*interruptFCN)(void), Uint16 qep_us, PCRM pcrm, 
	Uint16 lines, Uint16 polePairs);
static inline void StoreQEPConf(volatile struct QEP* qep, 
	volatile struct EQEP_REGS* eQEPRegs,Uint16 lines, Uint16 polePairs);
static inline void ConfigureEQEP(volatile struct EQEP_REGS* eQEPRegs, 
	void(*interruptFCN)(void), Uint32 qUPRD, PCRM pcrm, Uint32 qPOSMAX);
void InitEQep1Gpio(void);
void InitEQep2Gpio(void);


// ----------------------------------------------------------------------------
// Functions
// ----------------------------------------------------------------------------
void ConfigureQEP1(void(*interruptFCN)(void), Uint16 qep_us, PCRM pcrm, 
	Uint16 lines, Uint16 polePairs)
{
	Uint16 PRRSC = 1;
	// if additional preescaler is needed --> QCAPCTL (spru790a.pdf, pag. 33)
	// PRRSC << EQep1Regs.QCAPCTL.bit.CCPS;
	// enable peripheral clock
	EALLOW;
	SysCtrlRegs.PCLKCR1.bit.EQEP1ENCLK=1;
	EDIS;
	// configure inputs to be used as A-B-Z encoder signals
	InitEQep1Gpio();
	ConfigureEQEP(&EQep1Regs, interruptFCN, US2CLOCK(qep_us, PRRSC), pcrm,
		(lines << 2) -1);
	// store configuration
	StoreQEPConf(&qep1, &EQep1Regs, lines, polePairs);
}

// ----------------------------------------------------------------------------

void ConfigureQEP2(void(*interruptFCN)(void), Uint16 qep_us, PCRM pcrm, 
	Uint16 lines, Uint16 polePairs)
{
	Uint16 PRRSC = 1;
	// if additional preescaler is needed --> QCAPCTL (spru790.pdf, pag. 33)
	//PRRSC << EQep2Regs.QCAPCTL.bit.CCPS;
	// enable clock
	EALLOW;
	SysCtrlRegs.PCLKCR1.bit.EQEP2ENCLK=1;
	EDIS;
	// configure inputs to be used as A-B-Z encoder signals
	InitEQep2Gpio();
	ConfigureEQEP(&EQep2Regs, interruptFCN, US2CLOCK(qep_us, PRRSC), pcrm,
		(lines << 2) -1);
	// store configuration
	StoreQEPConf(&qep2, &EQep2Regs, lines, polePairs);
}

// ----------------------------------------------------------------------------


static inline void StoreQEPConf(volatile struct QEP* qep, 
	volatile struct EQEP_REGS* eQEPRegs,Uint16 lines, Uint16 polePairs)
{
	qep->eQEPRegs = eQEPRegs;
        qep->lines = lines;
        qep->polePairs = polePairs;
        qep->lastPos = 0;
        qep->theta_mech = 0;
        qep->theta_elec = 0;
        qep->w_mech = 0;
	qep->w_elec = 0;
	// 	calculate Hz/lines for the velocity loop. In real time we calculate the
	//	velocity as the difference in counts every time the Unit Timer reach its value
	//	so, we have counts/unit_timer_interrupt. To get Hz we appy this transformation:
	//	1/(QUPRD*CCPS/Tcpu)*1/(lines*4) = CPUINHZ()/(QUPRD*CCPS)*1/(lines*4)
	// PABLO: If an encoder of more than 4095 lines is connected --> IQ scalling is needed
	qep->lines2Hz=_IQdiv( _IQ(CpuInHz()/
						  ( eQEPRegs->QUPRD*(1<<eQEPRegs->QCAPCTL.bit.CCPS) )),
						  _IQ(lines) ) >> 2;
}

// ----------------------------------------------------------------------------

static inline void ConfigureEQEP(volatile struct EQEP_REGS* eQEPRegs, 
	void(*interruptFCN)(void),Uint32 qUPRD, PCRM pcrm, Uint32 qPOSMAX)
{
	eQEPRegs->QDECCTL.bit.QSRC=00;				// QEP quadrature count mode
	eQEPRegs->QEPCTL.bit.FREE_SOFT=2;			// avoid simulation reset
	eQEPRegs->QCAPCTL.bit.CCPS=0;				// eQEP capture timer clock prescaler
	eQEPRegs->QUPRD = qUPRD;					// configure Unit Timer period counter
	eQEPRegs->QEPCTL.bit.PCRM=pcrm;				// if 00 mode - QPOSCNT reset on index event
	eQEPRegs->QEPCTL.bit.UTE=1; 				// Unit Timer Enable 
	eQEPRegs->QCAPCTL.bit.UPPS=0;				// Unit position event prescaler
	eQEPRegs->QEPCTL.bit.QCLM=1; 				// Latch on unit timer timeout
	eQEPRegs->QPOSCNT=0;						// Reset counter
	eQEPRegs->QPOSMAX=qPOSMAX;					// Maximum position
	if(interruptFCN != 0){
    	EALLOW;
    	//eQEPRegs->QEINT.bit.IEL=1;			// enable interrupt on index event
		eQEPRegs->QCLR.bit.INT=1;				// clear all interrupt flags
		eQEPRegs->QCLR.bit.UTO=1;				// clear unit timer interrupt flag
    	eQEPRegs->QEINT.all=0;					// disable all interrupts
    	eQEPRegs->QEINT.bit.UTO=1;				// enable interrupt on unit time out
		if (eQEPRegs == &EQep1Regs) {
   			PieVectTable.EQEP1_INT = interruptFCN;	// assign ISR
   			EDIS;
       		PieCtrlRegs.PIEIER5.bit.INTx1 = 1;		// Enable EQEP1_INT in PIE
			IER |= M_INT5;							// Enable CPU Interrupt 5 
		}
		else {
   			PieVectTable.EQEP2_INT = interruptFCN;	// ISR
   			EDIS;
       		PieCtrlRegs.PIEIER5.bit.INTx2 = 1;		// Enable EQEP2_INT in PIE
			IER |= M_INT5;							// Enable CPU Interrupt 5 
		}
	}
}

// ----------------------------------------------------------------------------

void InitEQep1Gpio(void)
{
	ConfigureGPIOPort(_GPIO20, GPIO_INPUT, PERPH_1, GPIO_PULL_EN, GPIO_LOW);// EQEP1A
	ConfigureGPIOPort(_GPIO21, GPIO_INPUT, PERPH_1, GPIO_PULL_EN, GPIO_LOW);// EQEP1B
	ConfigureGPIOPort(_GPIO23, GPIO_INPUT, PERPH_1, GPIO_PULL_EN, GPIO_LOW);// EQEP1I
}

// ----------------------------------------------------------------------------

void InitEQep2Gpio(void)
{
	ConfigureGPIOPort(_GPIO24, GPIO_INPUT, PERPH_1, GPIO_PULL_EN, GPIO_LOW);// EQEP2A
	ConfigureGPIOPort(_GPIO25, GPIO_INPUT, PERPH_1, GPIO_PULL_EN, GPIO_LOW);// EQEP2B
	ConfigureGPIOPort(_GPIO26, GPIO_INPUT, PERPH_1, GPIO_PULL_EN, GPIO_LOW);// EQEP2I
}

// ----------------------------------------------------------------------------

void SwapDirection(volatile struct QEP *qep)
{
        qep->eQEPRegs->QDECCTL.bit.SWAP = 1;
}

// ----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------

