/******************************************************************************
** Copyright (C) 2010  
** Silvia Izquierdo Rosas <silvia.izquierdo.rosas@gmail.com>
** Christian Blanco Charro <khristian996@gmail.com>
** 
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.

** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.

** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

/**\file**********************************************************************
 File Name:   		sci.h
 Project:     		TIMotorLIB
 
 TI Doc: 		<a href="http://www.ti.com/litv/pdf/spru051d"> SPRU051B</a>
 \author  		Christian Blanco Charro (blancocristian@uniovi.es), 
			Silvia Izquierdo Rosas (silvia.izquierdo.rosas@gmail.com)
			Pablo Garcia (pgarcia@isa.uniovi.es)
 \date 			2010	      
 \brief 		SCI Routines
******************************************************************************/

//-----------------------------------------------------------------------------
//	Define
//-----------------------------------------------------------------------------
#ifndef SCI_H
#define SCI_H
/*#define	TAM_DATO 4
#define	TAM_NOMBRE_VAR 25
#define	TAM_LISTA_VAR 15
#define TAM_SERIAL_BUFFER (5*20)
*/

//-----------------------------------------------------------------------------
//	typedefs
// ----------------------------------------------------------------------------

/// SCI PORT configuration values
typedef enum {
	SCIA=0, ///< SCI A
	SCIB=1,	///< SCI B
	SCIC=2	///< SCI C
    } SCI_PORT;
    
/// number of stop bits configuration values
typedef enum {
	ONE_STOP_BIT=0, ///< one stop bit
	TWO_STOP_BIT=1	///< two stop bits
    } SCI_STOPBIT;
    
/// Parity configuration values
/// \todo: change this enum to have a new member NONE_PARITY and avoid the next enum (SCI_PARITY_ENABLE)
typedef enum {
	ODD_PARITY=0, 	///< odd parity
	EVEN_PARITY=1	///< even parity
    } SCI_PARITY;
    
/// Enable/Disable parity configuration values
typedef enum {
	PARITY_DISABLED=0, ///< parity disabled
	PARITY_ENABLED=1   ///< parity enabled
} SCI_PARITY_ENABLE; 

/*
/// Variable Description
struct VAR_DESC {
	char VarName[TAM_NOMBRE_VAR];	///< variable name
	void *PVar;			///< variable pointer
};*/ 
/// \todo: Pablo: A variable of VAR_DESC must be declared in this module

// ----------------------------------------------------------------------------
//Implemented Functions
// ----------------------------------------------------------------------------
void ConfigureSCI(interruptFCN, 
	SCI_PORT SciPort, Uint32 speed, SCI_STOPBIT StopBit , SCI_PARITY parity, 
	SCI_PARITY_ENABLE SciParityEnable, Uint16 nbit);
void SciFifoInit(volatile struct SCI_REGS* sciRegs);
void InitSCIGpio(SCI_PORT sciPort);
void EnableSCITXInterrupt(SCI_PORT sciPort, interruptFCN, unsigned char fifoLevel);
void DisableSCITXInterrupt(SCI_PORT sciPort);
//int AddVar2List(char *name,void *pointer,int indiceLeidas,struct VAR_DESC *variables); // DEPRECATED
//void ResetVector(Uint16* begin, Uint16* end);
//interrupt void SciaRxFifoIsr(void); // DEPRECATED
static inline void SCIARxPIEAck(void);
static inline void SCIATxPIEAck(void);
// transmit data
static inline void SciXmit(volatile struct SCI_REGS* sciRegs, int a);
static inline void SciMsg(volatile struct SCI_REGS* sciRegs, char *msg);
static inline void SendDataChar(volatile struct SCI_REGS* sciRegs, volatile char *a);
static inline void SendDataCharNow(volatile struct SCI_REGS* sciRegs, volatile char *a);
static inline void SendDataInt(volatile struct SCI_REGS* sciRegs, volatile Uint16 *a);
static inline void SendDataIntNow(volatile struct SCI_REGS* sciRegs, volatile Uint16 *a);
static inline void SendDataIQ(volatile struct SCI_REGS* sciRegs, volatile Uint32 *a);
static inline void SendDataIQNow(volatile struct SCI_REGS* sciRegs, volatile Uint32 *a);
// transmit data (SCIA)
static inline void SciaXmit(int a);
static inline void SciaMsg(char *msg);
static inline void SendDataCharSciA(volatile char *a);
static inline void SendDataCharSciANow(volatile char *a);
static inline void SendDataIntSciA(volatile Uint16 *a);
static inline void SendDataIntSciANow(volatile Uint16 *a);
static inline void SendDataIQSciA(volatile Uint32 *a);
static inline void SendDataIQSciANow(volatile Uint32 *a);
// transmit data (SCIB)
static inline void ScibXmit(int a);
static inline void ScibMsg(char *msg);
static inline void SendDataCharSciB(volatile char *a);
static inline void SendDataCharSciBNow(volatile char *a);
static inline void SendDataIntSciB(volatile Uint16 *a);
static inline void SendDataIntSciBNow(volatile Uint16 *a);
static inline void SendDataIQSciB(volatile Uint32 *a);
static inline void SendDataIQSciBNow(volatile Uint32 *a);
	
// ----------------------------------------------------------------------------

static inline void SCIARxPIEAck(void)
{
	SciaRegs.SCIFFRX.bit.RXFFOVRCLR = 1;   // Clear Overflow flag
	SciaRegs.SCIFFRX.bit.RXFFINTCLR = 1;   // Clear Interrupt flag
	PieCtrlRegs.PIEACK.all |= PIEACK_GROUP9;       // Issue PIE ack
}

// ----------------------------------------------------------------------------

static inline void SCIATxPIEAck(void)
{
	SciaRegs.SCIFFTX.bit.TXFFINTCLR = 1;   // Clear Interrupt flag
	PieCtrlRegs.PIEACK.all |= PIEACK_GROUP9;       // Issue PIE ack
}

// ----------------------------------------------------------------------------

static inline void SciXmit(volatile struct SCI_REGS* sciRegs, int a)
{
    while (sciRegs->SCIFFTX.bit.TXFFST != 0) {}	// si TXFFST=0 -> fifo is empty, ready to transmit TODO: Pablo Interrupts??
    sciRegs->SCITXBUF=a;	
}

// ----------------------------------------------------------------------------

static inline void SciMsg(volatile struct SCI_REGS* sciRegs, char *msg)
{
	int i=0;
    while(msg[i] != '\0') {
        SciXmit(sciRegs, msg[i]);
        i++;
    }
}

// ----------------------------------------------------------------------------

static inline void SendDataChar(volatile struct SCI_REGS* sciRegs, volatile char *a)
{
	//Receives a pointer to the number  
	while (sciRegs->SCIFFTX.bit.TXFFST != 0) {}	// si TXFFST=0 -> fifo is empty, ready to transmit TODO: Pablo Interrupts??
	sciRegs->SCITXBUF = *a;	
}

// ----------------------------------------------------------------------------

static inline void SendDataInt(volatile struct SCI_REGS* sciRegs, volatile Uint16 *a)
{
	//Receives a pointer to the number  
	int i;
	for(i=0;i<2;i++) {	
	    while (sciRegs->SCIFFTX.bit.TXFFST != 0) {}	// si TXFFST=0 -> fifo is empty, ready to transmit TODO: Pablo Interrupts??	    
	    sciRegs->SCITXBUF = (*a >> 8*i & 0x00FF);
	}
}

// ----------------------------------------------------------------------------

static inline void SendDataIQ(volatile struct SCI_REGS* sciRegs, volatile Uint32 *a)
{
	//Receives a pointer to the number  
	int i;
	for(i=0;i<4;i++) {	
	    while (sciRegs->SCIFFTX.bit.TXFFST != 0) {}	// si TXFFST=0 -> fifo is empty, ready to transmit TODO: Pablo Interrupts??	    
	    sciRegs->SCITXBUF = (*a >> 8*i & 0x00FF);
	}	
}

// ----------------------------------------------------------------------------

static inline void SendDataCharNow(volatile struct SCI_REGS* sciRegs, volatile char *a)
{
	//Receives a pointer to the number  
	sciRegs->SCITXBUF = *a;	
}

// ----------------------------------------------------------------------------

static inline void SendDataIntNow(volatile struct SCI_REGS* sciRegs, volatile Uint16 *a)
{
	//Receives a pointer to the number  
	int i;
	for(i=0;i<2;i++) {	
	    sciRegs->SCITXBUF = (*a >> 8*i & 0x00FF);
	}
}

// ----------------------------------------------------------------------------

static inline void SendDataIQNow(volatile struct SCI_REGS* sciRegs, volatile Uint32 *a)
{
	//Receives a pointer to the number  
	int i;
	for(i=0;i<4;i++) {
	    sciRegs->SCITXBUF = (*a >> 8*i & 0x00FF);
	}	
}

// ----------------------------------------------------------------------------

/// \deprecated: ?? Sends a character through serial port
/// \todo: Interrupt driven code
static inline void SciaXmit(int a)
{
	SciXmit(&SciaRegs,a);
}

// ----------------------------------------------------------------------------

//Sends a message through serial port
static inline void SciaMsg(char * msg)
{
	SciMsg(&SciaRegs,msg);
}

// ----------------------------------------------------------------------------

/// \todo: Interrupt driven code
static inline void SendDataCharSciA(volatile char *a) //8bits
{
	SendDataChar(&SciaRegs, a);

}

// ----------------------------------------------------------------------------

/// \todo: Interrupt driven code
static inline void SendDataIntSciA(volatile Uint16 *a)
{
	SendDataInt(&SciaRegs, a);
} 

// ----------------------------------------------------------------------------

/// \todo: Interrupt driven code
static inline void SendDataIQSciA(volatile Uint32 *a)
{
	SendDataIQ(&SciaRegs, a);
}

// ----------------------------------------------------------------------------

static inline void SendDataCharSciANow(volatile char *a) //8bits
{
	SendDataCharNow(&SciaRegs, a);
}

// ----------------------------------------------------------------------------

static inline void SendDataIntSciANow(volatile Uint16 *a)
{
	SendDataIntNow(&SciaRegs, a);
} 

// ----------------------------------------------------------------------------

static inline void SendDataIQSciANow(volatile Uint32 *a)
{
	SendDataIQNow(&SciaRegs, a);
}

// ----------------------------------------------------------------------------

/// \deprecated: ?? Sends a character through serial port
/// \todo: Interrupt driven code
static inline void ScibXmit(int a)
{
	SciXmit(&ScibRegs,a);
}

// ----------------------------------------------------------------------------

//Sends a message through serial port
static inline void ScibMsg(char * msg)
{
	SciMsg(&ScibRegs,msg);
}

// ----------------------------------------------------------------------------

/// \todo: Interrupt driven code
static inline void SendDataCharSciB(volatile char *a) //8bits
{
	SendDataChar(&ScibRegs, a);

}

// ----------------------------------------------------------------------------

/// \todo: Interrupt driven code
static inline void SendDataIntSciB(volatile Uint16 *a)
{
	SendDataInt(&ScibRegs, a);
} 

// ----------------------------------------------------------------------------

/// \todo: Interrupt driven code
static inline void SendDataIQSciB(volatile Uint32 *a)
{
	SendDataIQ(&ScibRegs, a);
}

// ----------------------------------------------------------------------------

static inline void SendDataCharSciBNow(volatile char *a) //8bits
{
	SendDataCharNow(&ScibRegs, a);
}

// ----------------------------------------------------------------------------

static inline void SendDataIntSciBNow(volatile Uint16 *a)
{
	SendDataIntNow(&ScibRegs, a);
} 

// ----------------------------------------------------------------------------

static inline void SendDataIQSciBNow(volatile Uint32 *a)
{
	SendDataIQNow(&ScibRegs, a);
}

// ----------------------------------------------------------------------------

#endif
//-----------------------------------------------------------------------------
// End Of File
//----------------------------------------------------------------------------- 
